from emod_api import schema_to_class as s2c
from emod_api.interventions import utils

schema_path = None
iv_name = "MCV1"
#dupe_policy = "Replace" # or "Add" or "Abort" -- from covid branch
# Note that duration (what we call waning profile) needs to be configurable, but in an intuitive way

def _SimpleMCV( camp, timestep, coverage=1.0, age_at_vacc_in_days=1, mv_name=iv_name, nodes=None ):
    """
    MCV1 Campaign
    :param timestep: When? Timestep to distribute vaccine on.
    :param coverage: Who? Demographic Coverage.
    :param age_at_vacc_in_days: Who/When? Delay period constant for intervention. This controls the age at which people get the vaccnation.
    :param mv_name: What? This doesn't matter unless you want to be able to give up multiple vaccines to individuals but have them appear to be different, avoid de-duplication issues.
    :param nodes: Where? Optimally specific the ids of the geographic nodes to be included in this distribution.
    """
    # First, get the objects
    event = s2c.get_class_with_defaults( "CampaignEvent", schema_path )
    coordinator = s2c.get_class_with_defaults( "StandardEventCoordinator", schema_path )
    if coordinator is None:
        print( "s2c.get_class_with_defaults returned None. Maybe no schema.json was provided." )
        return ""

    intervention = s2c.get_class_with_defaults( "Vaccine", schema_path )
    efficacy_profile = "WaningEffectConstant"
    waning = s2c.get_class_with_defaults( efficacy_profile, schema_path )
    waning.Initial_Effect = 0.999

    meta_meta_intervention = s2c.get_class_with_defaults( "DelayedIntervention", schema_path )
    meta_meta_intervention.Actual_IndividualIntervention_Configs = [ intervention ]
    meta_meta_intervention.Delay_Period_Distribution="CONSTANT_DISTRIBUTION"
    meta_meta_intervention.Delay_Period_Constant=age_at_vacc_in_days

    meta_intervention = s2c.get_class_with_defaults( "NodeLevelHealthTriggeredIV", schema_path )
    meta_intervention.Demographic_Coverage=coverage
    meta_intervention.Duration=-1
    meta_intervention.Trigger_Condition_List=['Births']
    meta_intervention.Actual_IndividualIntervention_Config = meta_meta_intervention

    # Second, hook them up
    #  event->coordinator->meta_intervention(nlhti)->meta_meta_intervention(delay)->intervention(vaccine)

    event.Event_Coordinator_Config = coordinator
    coordinator.Intervention_Config = meta_intervention
    intervention.Acquire_Config = waning 
    event.Start_Day = float(timestep+1)

    # Third, do the actual settings
    intervention.Intervention_Name = mv_name

    if nodes and len(nodes) > 0:
        event.Nodeset_Config = utils.do_nodes( camp.schema_path, nodes )

    event["Name"] = "MCV1" # ???

    return event

def SimpleMCV1( camp, timestep, coverage=1.0, nodes=None ):
    if camp is not None:
        global schema_path
        schema_path = camp.schema_path
    return _SimpleMCV( camp, timestep, coverage, age_at_vacc_in_days=270, mv_name="MCV1", nodes=nodes )

def SimpleMCV2( camp, timestep, coverage=1.0, nodes=None):
    if camp is not None:
        global schema_path
        schema_path = camp.schema_path
    return _SimpleMCV( camp, timestep, coverage, age_at_vacc_in_days=475, mv_name="MCV2", nodes=nodes )

def new_intervention_as_file( camp, timestep, filename=None ):
    camp.add( _SimpleMCV( camp, timestep ) )
    if filename is None:
        filename = "measles_vaccine.json"
    camp.save( filename )
    return filename

