#!/usr/bin/env python
import unittest
import json
import os
import emodpy_measles.interventions.complex_import as ci
import emod_api.campaign as camp
import emodpy_measles.interventions.targeted_outbreak as target_outbreak

import emod_api.campaign as camp
import emodpy_measles.interventions.simple_sia as sia
import emodpy_measles.interventions.measles_vaccine as mv
import emodpy_measles.interventions.complex_import as ci


class TestCampInterventions(unittest.TestCase):

    is_debugging = True

    def validate_measles_vaccine(self):
        target_event = self.campaign['Events'][0]
        start_day = target_event['Start_Day']
        intervention_name = target_event['Event_Coordinator_Config']['Intervention_Config']['Intervention_Name']
        node_config = self.campaign['Events'][0]['Nodeset_Config']
        coverage = target_event["Event_Coordinator_Config"]["Intervention_Config"]["Demographic_Coverage"]

        self.assertEqual(start_day, self.timestep + 1)
        self.assertEqual(intervention_name, "NodeLevelHealthTriggeredIV")
        self.assertEqual(node_config, self.node_config)
        self.assertEqual(coverage, self.coverage)

    def test_simplemcv1_default(self):
        camp.schema_path = "schema.json"

        # testing that intervention as file creates file
        try:
            camp_filename = mv.new_intervention_as_file( camp, 10 )
            print( f"Check for valid campaign file at: {camp_filename}." )
        except ValueError as ex:
            print( "Caught exception invoking simple_vaccine.new_intervention_as_file(10) -- " + str( ex.strerror ) )
        except Exception as ex:
            print( "Caught exception invoking simple_vaccine.new_intervention_as_file(10) -- " + str( ex ) )

        self.coverage = 1
        self.timestep = 1
        self.node_config = {"class": "NodeSetAll"}

        event = mv.SimpleMCV1(camp, timestep=1)
        camp.add(event, first=True)
        camp.add(mv.SimpleMCV1(camp, timestep=10))

        camp_file = "campaign_simple_mcv1.json"
        camp.save( camp_file )

        self.assertTrue(os.path.exists(camp_file))
        
        with open(camp_file, 'r') as file:
            self.campaign = json.load(file)
        
        # Check that events are being added properly
        self.assertEqual(len(self.campaign['Events']), 2)
        self.validate_measles_vaccine()

        if not self.is_debugging:
            os.remove(camp_file)

    def test_simplemcv1_custom(self):
        camp.schema_path = "schema.json"

        # testing that intervention as file creates file
        try:
            camp_filename = mv.new_intervention_as_file( camp, 20 )
            print( f"Check for valid campaign file at: {camp_filename}." )
        except ValueError as ex:
            print( "Caught exception invoking simple_vaccine.new_intervention_as_file(20) -- " + str( ex.strerror ) )
        except Exception as ex:
            print( "Caught exception invoking simple_vaccine.new_intervention_as_file(20) -- " + str( ex ) )

        self.coverage = 0.4
        self.timestep = 5
        nodes = [1, 3]
        self.node_config = {'Node_List': nodes, 'class': 'NodeSetNodeList'}

        event = mv.SimpleMCV1(camp, coverage=self.coverage, timestep=self.timestep, nodes=nodes)
        camp.add(event, first=True)

        camp_file = "campaign_simple_mcv1.json"
        camp.save( camp_file )

        self.assertTrue(os.path.exists(camp_file))
        
        with open(camp_file, 'r') as file:
            self.campaign = json.load(file)

        self.validate_measles_vaccine()

        if not self.is_debugging:
            os.remove(camp_file)

    def test_simplemcv2_default(self):
        import emod_api.campaign as camp
        import emodpy_measles.interventions.measles_vaccine as mv
        camp.schema_path = "schema.json"


        # testing that intervention as file creates file
        try:
            camp_filename = mv.new_intervention_as_file( camp, 10 )
            print( f"Check for valid campaign file at: {camp_filename}." )
        except ValueError as ex:
            print( "Caught exception invoking simple_vaccine.new_intervention_as_file(10) -- " + str( ex.strerror ) )
        except Exception as ex:
            print( "Caught exception invoking simple_vaccine.new_intervention_as_file(10) -- " + str( ex ) )

        self.timestep = 1
        self.node_config = {"class": "NodeSetAll"}
        self.coverage = 1

        event = mv.SimpleMCV2(camp, timestep=1)
        camp.add(event, first=True)
        camp.add(mv.SimpleMCV2(camp, timestep=10))

        camp_file = "campaign_simple_mcv2.json"
        camp.save( camp_file )

        self.assertTrue(os.path.exists(camp_file))
        
        with open(camp_file, 'r') as file:
            self.campaign = json.load(file)
        
        # Check that events are being added properly
        self.assertEqual(len(self.campaign['Events']), 2)
        self.validate_measles_vaccine()

        if not self.is_debugging:
            os.remove(camp_file)

    def test_simplemcv2_custom(self):
        import emodpy_measles.interventions.measles_vaccine as mv
        import emod_api.campaign as camp
        camp.schema_path = "schema.json"

        # testing that intervention as file creates file
        try:
            camp_filename = mv.new_intervention_as_file( camp, 20 )
            print( f"Check for valid campaign file at: {camp_filename}." )
        except ValueError as ex:
            print( "Caught exception invoking simple_vaccine.new_intervention_as_file(20) -- " + str( ex.strerror ) )
        except Exception as ex:
            print( "Caught exception invoking simple_vaccine.new_intervention_as_file(20) -- " + str( ex ) )

        self.timestep = 7
        self.coverage = 0.6

        nodes = [1, 3]
        self.node_config = {'Node_List': nodes, 'class': 'NodeSetNodeList'}
        event = mv.SimpleMCV2(camp, coverage=self.coverage, timestep=self.timestep, nodes=nodes)
        camp.add(event, first=True)

        camp_file = "campaign_simple_mcv2.json"
        camp.save( camp_file )

        self.assertTrue(os.path.exists(camp_file))
        
        with open(camp_file, 'r') as file:
            self.campaign = json.load(file)

        self.validate_measles_vaccine()

        if not self.is_debugging:
            os.remove(camp_file)

    def validate_complex_import(self):
        intervention_config = self.campaign['Events'][0]['Event_Coordinator_Config']['Intervention_Config']
        dips = intervention_config['Daily_Import_Pressures']
        durations = intervention_config['Durations']
        start_day = self.campaign['Events'][0]['Start_Day']
        node_config = self.campaign['Events'][0]['Nodeset_Config']

        self.assertEqual(dips, self.dips)
        self.assertEqual(start_day, self.timestep + 1)
        self.assertEqual(durations, self.durations)
        self.assertEqual(node_config, self.node_config)

    def test_complex_import_default(self):
        camp.schema_path = "schema.json"

        try:
            camp_filename = ci.new_intervention_as_file( camp, timestep=15 )
            print( f"Check for valid campaign file at: {camp_filename}." )
        except ValueError as ex:
            print( "Caught exception invoking complex_import.new_intervention_as_file(15) -- " + str( ex.strerror ) )
        except Exception as ex:
            print( "Caught exception invoking complex_import.new_intervention_as_file(15) -- " + str( ex ) )

        self.durations  = [100000]
        self.dips =[1/(774*10)]
        self.timestep=1
        self.node_config = {"class": "NodeSetAll"}

        event = ci.ComplexImportationEvent( camp )
        camp.add(event, first=True)

        camp_file = "campaign_complex_import.json"
        camp.save( camp_file )
        self.assertTrue(os.path.exists(camp_file))
        
        with open(camp_file, 'r') as file:
            self.campaign = json.load(file)

        self.validate_complex_import()

        if not self.is_debugging:
            os.remove(camp_file)

    def test_complex_import_custom(self):
        camp.schema_path = "schema.json"

        try:
            camp_filename = ci.new_intervention_as_file( camp, timestep=5 )
            print( f"Check for valid campaign file at: {camp_filename}." )
        except ValueError as ex:
            print( "Caught exception invoking complex_import.new_intervention_as_file(5) -- " + str( ex.strerror ) )
        except Exception as ex:
            print( "Caught exception invoking complex_import.new_intervention_as_file(5) -- " + str( ex ) )

        self.durations  = [10, 10, 11, 15]
        self.dips =[10, 10, 11, 7]
        self.timestep=10
        nodes = [1, 5]
        self.node_config = {'Node_List': nodes, 'class': 'NodeSetNodeList'}
        event = ci.ComplexImportationEvent(camp, durs=self.durations, dips=self.dips, timestep=10, nodes=nodes)
        camp.add(event, first=True)

        camp_file = "campaign_complex_import.json"
        camp.save( camp_file )
        self.assertTrue(os.path.exists(camp_file))
        
        with open(camp_file, 'r') as file:
            self.campaign = json.load(file)

        self.validate_complex_import()

        if not self.is_debugging:
            os.remove(camp_file)

    def test_complex_import_errorhandling(self):
        with self.assertRaises(ValueError):
            ci.ComplexImportationEvent(camp, durs=[], dips=[10], timestep=10)
        with self.assertRaises(ValueError):
            ci.ComplexImportationEvent(camp, durs=[10], dips=[], timestep=10)
        with self.assertRaises(ValueError):
            ci.ComplexImportationEvent(camp, durs=[10, 11], dips=[10], timestep=10)

    def validate_sia(self):
        event_coord = self.campaign['Events'][0]['Event_Coordinator_Config']
        Target_Age_Min = event_coord['Target_Age_Min']
        Target_Age_Max = event_coord['Target_Age_Max']
        start_day = self.campaign['Events'][0]['Start_Day']
        node_config = self.campaign['Events'][0]['Nodeset_Config']

        self.assertEqual(self.Target_Age_Min, Target_Age_Min)
        self.assertEqual(self.Target_Age_Max, Target_Age_Max)
        self.assertEqual(self.timestep+1, start_day)
        self.assertEqual(node_config, self.node_config)

    def test_sia_default(self):
        camp.schema_path = "schema.json"

        try:
            camp_filename = sia.new_intervention_as_file( camp, 10 )
            print( f"Check for valid campaign file at: {camp_filename}." )
        except ValueError as ex:
            print( "Caught exception invoking sia.new_intervention_as_file(10) -- " + str( ex.strerror ) )
        except Exception as ex:
            print( "Caught exception invoking sia.new_intervention_as_file(10) -- " + str( ex ) )

        self.timestep=1
        self.Target_Age_Min=0.75
        self.Target_Age_Max=5.0
        self.node_config = {"class": "NodeSetAll"}

        event = sia.SimpleSIA( camp )
        camp.add( event, first=True )

        camp_file = "campaign_simple_sia.json"
        camp.save( camp_file )

        with open(camp_file, 'r') as file:
            self.campaign = json.load(file)
        
        self.validate_sia()

        if not self.is_debugging:
            os.remove(camp_file)

    def test_sia_custom(self):
        camp.schema_path = "schema.json"

        try:
            camp_filename = sia.new_intervention_as_file( camp, 10 )
            print( f"Check for valid campaign file at: {camp_filename}." )
        except ValueError as ex:
            print( "Caught exception invoking sia.new_intervention_as_file(10) -- " + str( ex.strerror ) )
        except Exception as ex:
            print( "Caught exception invoking sia.new_intervention_as_file(10) -- " + str( ex ) )

        self.timestep=1
        self.Target_Age_Min=0.75
        self.Target_Age_Max=5.0
        nodes = [3, 4]
        self.node_config = {'Node_List': nodes, 'class': 'NodeSetNodeList'}

        event = sia.SimpleSIA(camp, timestep=self.timestep, Target_Age_Min=self.Target_Age_Min,
                              Target_Age_Max=self.Target_Age_Max, nodes=nodes)
        camp.add( event, first=True )

        camp_file = "campaign_simple_sia.json"
        camp.save( camp_file )
        
        with open(camp_file, 'r') as file:
            self.campaign = json.load(file)
        
        self.validate_sia()

        if not self.is_debugging:
            os.remove(camp_file)

    def validate_target_outbreak(self):
        event_coord = self.campaign['Events'][0]['Event_Coordinator_Config']

        coverage = event_coord['Demographic_Coverage']
        start_day = self.campaign['Events'][0]['Start_Day']

        if isinstance(self.node_ids, str):
            node_ids = self.campaign['Events'][0]['Nodeset_Config']['class']
        else:
            node_ids = self.campaign['Events'][0]['Nodeset_Config']['Node_List']

        self.assertEqual(self.node_ids, node_ids)
        self.assertEqual(self.coverage, coverage)
        self.assertEqual(self.timestep, start_day)

    
    def test_target_outbreak_default(self):
        camp.schema_path = "schema.json"
        self.timestep = 5
        self.coverage = 0.01
        self.node_ids = "NodeSetAll"

        event = target_outbreak.seed(camp, timestep=self.timestep)
        camp.add(event, first=True)

        camp_file = "campaign_target_outbreak.json"
        camp.save(camp_file)

        with open(camp_file, 'r') as file:
            self.campaign = json.load(file)

        self.validate_target_outbreak()

    def test_target_outbreak_custom(self):
        camp.schema_path = "schema.json"
        self.timestep = 20
        self.coverage = 0.5
        self.node_ids = [1, 2]

        event = target_outbreak.seed(camp, timestep=self.timestep, 
                                     coverage=self.coverage, 
                                     node_ids=self.node_ids)
        camp.add(event, first=True)

        camp_file = "campaign_target_outbreak.json"
        camp.save(camp_file)

        with open(camp_file, 'r') as file:
            self.campaign = json.load(file)

        self.validate_target_outbreak()

        if not self.is_debugging:
            os.remove(camp_file)

if __name__ == '__main__':
    unittest.main()
